//---------------------------------------------------------------------------
#pragma hdrstop
#include "W25Qxx.h"             //     
#include "ProgressBar.h"        //   -
#include "MainProgram.h"        //    
#include "ChipFunctions.h"      //      
#include "PortFunctions.h"      //    
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---    (  WEL) --------------------------
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_WriteEnable()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 

  //--- WRITE ENABLE ---
  //    1 
  PaketSize=1;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x08=8  = 1  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  0x06 - write enable,     
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=0x06;                                  //  Write Enable
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  return true;
}
//---------------------------------------------------------------------------
//---    (  WEL) ----------------------------
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_WriteDisable()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 

  //--- WRITE DISABLE ---
  //    1 
  PaketSize=1;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x08=8  = 1  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  0x04 - write disable,     
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=0x04;                                  //  Write Disable
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  return true;
}
//---------------------------------------------------------------------------
//---  STATUS1, STATUS2 ( volatile bits) ------------------------
//--- STATUS1: SRP0, SEC, TB, BP2, BP1, BP0 (bits 7-2) ----------------------
//--- STATUS2: CMP, LB3, LB2, LB1, QE, SRP1 (bits 14-8) ---------------------
//--- SRP1, LB3, LB2, LB1 can be set to 1, but can't be cleared back to 0 ---
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_WriteStatus(unsigned char Status1, unsigned char Status2)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 

  //--- WRITE STATUS ---
  //    3 
  PaketSize=3;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x18=24  = 3  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //   ,     
  wr_buf.Size=4;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=Status2;                               // Status2
  wr_buf.Data[2]=Status1;                               // Status1
  wr_buf.Data[3]=0x01;                                  //  Write Status
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  Sleep(15);                                            //    ( 15 )
  return true;
}
//---------------------------------------------------------------------------
//---  STATUS1, STATUS2 -----------------------------------------------
//---------------------------------------------------------------------------
// output:      bool - success, unsigned char - Status1, Status2
W25Qxx_RS W25Qxx_ReadStatus()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 
  W25Qxx_RS OutputData;                 //   

  OutputData.Success=false;             //    
  //--- READ STATUS ---
  //    3 
  PaketSize=3;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x18=24  = 3  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //   ,     
  wr_buf.Size=4;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=0;                                     // Dummy Byte 2
  wr_buf.Data[2]=0;                                     // Dummy Byte 1
  wr_buf.Data[3]=0x05;                                  //  Read Status
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //     (3    SPI,  )
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x45;                                  // ReadPacket
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  //   
  rd_buf=ReadFrom();
  if(!rd_buf.Size){     SendMsgToLog(0x21,"");          //   
                        LBar->Free();                   //    -
                        ClosePort();
                        return OutputData;
                  }
  //     
  OutputData.Status1 = rd_buf.Data[1];
  OutputData.Status2 = rd_buf.Data[0];                  //    SPI    
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---    (4 kB) ---------------------------------------
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_EraseSector(unsigned int SectorAddress)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 

  //--- SECTOR ERASE ---
  //    4 
  PaketSize=4;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x08=8  = 1  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  0x20 (Sector Erase)     (24 bit)
  wr_buf.Size=5;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=SectorAddress;                         //   (    )
  wr_buf.Data[2]=SectorAddress>>8;                      //  
  wr_buf.Data[3]=SectorAddress>>16;                     //  
  wr_buf.Data[4]=0x20;                                  //  Erase Sector
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  Sleep(400);                                           //     ( 400 )
  return true;
}
//---------------------------------------------------------------------------
//---    (256 B) -----------------------------------
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_WritePage(unsigned int PageAddress)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 

  //--- PAGE WRITE ---
  //    4 
  PaketSize=4;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x20=32  = 4  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //---   0x02 (Page Program)     (24 bit)
  wr_buf.Size=5;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=PageAddress;                           //     
  wr_buf.Data[2]=PageAddress>>8;                        //   
  wr_buf.Data[3]=PageAddress>>16;                       //   
  wr_buf.Data[4]=0x02;                                  //  Page Program
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    8 
  PaketSize=8;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 64  = 8 ,  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  256 ,   
  for(unsigned int i=0;i<256/8;i++)
  {     //---  
        wr_buf.Size=9;         //  
        wr_buf.Data[0]=0x44;       //LoadPacket
        wr_buf.Data[1]=SelectedChip.Data[PageAddress+7]; //       , 
        wr_buf.Data[2]=SelectedChip.Data[PageAddress+6]; //        ,
        wr_buf.Data[3]=SelectedChip.Data[PageAddress+5]; //        
        wr_buf.Data[4]=SelectedChip.Data[PageAddress+4]; //    (       )
        wr_buf.Data[5]=SelectedChip.Data[PageAddress+3];
        wr_buf.Data[6]=SelectedChip.Data[PageAddress+2];
        wr_buf.Data[7]=SelectedChip.Data[PageAddress+1];
        wr_buf.Data[8]=SelectedChip.Data[PageAddress+0];
        if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");   //   
                               LBar->Free();            //    -
                               ClosePort();
                               return false;
                             }
        //   SPI
        wr_buf.Size=1;   //  
        wr_buf.Data[0]=0x46; //StartChange
        if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");   //   
                               LBar->Free();            //    -
                               ClosePort();
                               return false;
                             }
        //*******
        Sleep(2);                                       //     SPI
        //*******
        PageAddress=PageAddress+8;
  }
  // CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  Sleep(3);                                             //    
  return true;
}
//---------------------------------------------------------------------------
//---        --------------------
//---------------------------------------------------------------------------
// output:      bool - success
bool W25Qxx_ReadData(unsigned int StartAddress, unsigned int EndAddress)
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  unsigned int PaketSize;               //  , 
  unsigned int CurrentAddress;          //  , 
  unsigned int CurrentSector;           //  

  //    4 
  PaketSize=4;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 0x20=32  = 4  -  
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //  CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x47;                                  // Clear CS ()
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //---   0x03 (ReadData)     (24 bit)
  wr_buf.Size=5;                                        //   
  wr_buf.Data[0]=0x44;                                  // LoadPacket
  wr_buf.Data[1]=StartAddress;                          //     
  wr_buf.Data[2]=StartAddress>>8;                       //   
  wr_buf.Data[3]=StartAddress>>16;                      //   
  wr_buf.Data[4]=0x03;                                  //  Read Data
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    SPI
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x46;                                  // StartChange
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //    8 
  PaketSize=8;
  wr_buf.Size=2;                                        //   
  wr_buf.Data[0]=0x42;                                  // LoadPacketSize
  wr_buf.Data[1]=PaketSize*8;                           // 64  = 8 
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  //   
  for(CurrentAddress=StartAddress; CurrentAddress<(EndAddress+1); CurrentAddress=CurrentAddress+8)
  {       //   ( 8 )
          wr_buf.Size=1;                                //   
          wr_buf.Data[0]=0x46;                          // StartChange
          if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,""); //   
                                 LBar->Free();          //    -
                                 ClosePort();
                                 return false;
                               }
          //*******
          Sleep(2);                                     //     SPI
          //*******
          //  
          wr_buf.Size=1;                                //   
          wr_buf.Data[0]=0x45;                          // ReadPacket
          if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,""); //   
                                 LBar->Free();          //    -
                                 ClosePort();
                                 return false;
                               }
          // 
          rd_buf=ReadFrom();
          if(!rd_buf.Size){ SendMsgToLog(0x21,"");      //   
                            LBar->Free();               //    -
                            ClosePort();
                            return false;
                          }
          //    
          for(unsigned int i=0; i<PaketSize; i++)
          {     SelectedChip.Data[CurrentAddress+i]=rd_buf.Data[PaketSize-i-1]; //     
          }
          //   - (    )
          CurrentSector=CurrentAddress/4096;
          if((CurrentAddress-CurrentSector*4096)==0)
          {     LBar->ProgressBar1->Position=100*(CurrentAddress-StartAddress)/(EndAddress-StartAddress+1); //    -
          }
  }
  // CS   
  wr_buf.Size=1;                                        //   
  wr_buf.Data[0]=0x48;                                  // Set CS ( )
  if(!WriteTo(wr_buf)) { SendMsgToLog(0x11,"");         //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return false;
                       }
  return true;
}
//---------------------------------------------------------------------------
//---    STATUS1, STATUS2 -----------------------
//---------------------------------------------------------------------------
// output:      bool - success
void W25Qxx_StatusDecode(unsigned char Status1, unsigned char Status2)
{ //   BUSY
  if(Status1 && 0x1)            Form1->RichEdit1->Lines->Add("Bit BUSY is set. Chip is busy.");
  else                          Form1->RichEdit1->Lines->Add("Bit BUSY is cleared. Chip is redy for further instructions.");
  //   WEL
  if(Status1 && 0x2)            Form1->RichEdit1->Lines->Add("Bit WEL is set. Write is enabled.");
  else                          Form1->RichEdit1->Lines->Add("Bit WEL is cleared. Write is disabled.");
  //   TB, BP2, BP1, BP0
  if(Status1 && 0x3C)           Form1->RichEdit1->Lines->Add("One of TB, BP2, BP1, BP0 is not cleared. Memory protection is enabled.");
  else                          Form1->RichEdit1->Lines->Add("All of TB, BP2, BP1, BP0 are cleared. Memory protection is disabled.");
  //   SEC
  if(Status1 && 0x40)           Form1->RichEdit1->Lines->Add("Bit SEC is set. Sector protection is selected.");
  else                          Form1->RichEdit1->Lines->Add("Bit SEC is cleared. Block protection is selected.");
  //   SRP
  unsigned char SRP = (Status1>>7)|(Status2<<1);
  switch(SRP)
  {     case 0: Form1->RichEdit1->Lines->Add("Bits SRP1, SRP0 are cleared. Status register can be written at any time.");
                break;
        case 1: Form1->RichEdit1->Lines->Add("Bit SRP1 is cleared and bit SRP0 is set. Status register protection depends on the WP pin value.");
                break;
        case 2: Form1->RichEdit1->Lines->Add("Bit SRP1 is set and bit SRP0 is cleared. Status register is protected until the next power-down, power-up cycle.");
                break;
        case 3: Form1->RichEdit1->Lines->Add("Bits SRP1, SRP0 is set. Status register is permanently protected and can not be written to.");
                break;
        default: break;
  }
  //   QE
  if(Status2 && 0x2)            Form1->RichEdit1->Lines->Add("Bit QE is set. Quad SPI is enabled.");
  else                          Form1->RichEdit1->Lines->Add("Bit QE is cleared. Quad SPI is disabled.");
  //   LB3, LB2, LB1
  if(Status2 && 0x38)           Form1->RichEdit1->Lines->Add("One of LB3, LB2, LB1 is not cleared. Some of security registers are locked.");
  else                          Form1->RichEdit1->Lines->Add("All of LB3, LB2, LB1 are cleared. All of security registers are unlocked.");
  //   CMP
  if(Status2 && 0x40)           Form1->RichEdit1->Lines->Add("Bit CMP is set.");
  else                          Form1->RichEdit1->Lines->Add("Bit CMP is cleared.");
  //   SUS
  if(Status2 && 0x80)           Form1->RichEdit1->Lines->Add("Bit SUS is set.");
  else                          Form1->RichEdit1->Lines->Add("Bit SUS is cleared.");
}
//---------------------------------------------------------------------------


